@@ -1,4 +1,3 @@
-
 #' Outliergram for univariate functional datasets
 #'
 #' This function performs the outliergram of a univariate functional dataset,
@@ -24,16 +23,6 @@
 #' If missing, MBDs are computed.
 #' @param MEI_data a vector containing the MEI for each element of the dataset.
 #' If not not provided, MEIs are computed.
-#' @param q_low parameter used in the part where data are shifted toward the
-#' center of the dataset. It indicates the quantile to be used to compute the
-#' target to compare functions in the secondary check for outliers. Defult is 0,
-#' i.e. High MEI functions (lying at the bottom of the dataset) are compared
-#' to the minimum of all the remaining functions.
-#' @param q_high parameter used in the part where data are shifted toward the
-#' center of the dataset. It indicates the quantile to be used to compute the
-#' target to compare functions in the secondary check for outliers.
-#' Defult is 1, i.e. Low MEI functions (lying at the top of the dataset) are
-#' compared to the maximum of all the remaining functions.
 #' @param p_check percentage of observations with either low or high MEI to be
 #' checked for outliers in the secondary step (shift towards the center of the
 #' dataset).
@@ -89,9 +78,12 @@
 #' @return
 #'
 #' Even when used graphically to plot the outliergram, the function returns a
-#' list containing a numeric vector with the IDs of observations in
-#' \code{fData} that are considered as shape outliers and the value of
-#' \code{Fvalue} that has been used in determining them.
+#' list containing:
+#' \itemize{
+#' \item{\code{Fvalue}}{: the value of the parameter F used;}
+#' \item{\code{d}}{: the vector of values of the parameter \eqn{d} for each observation
+#' (distance to the parabolic border of the outliergram);}
+#' \item{\code{ID_outliers}}{: the vector of observations id corresponding to outliers.}}
 #'
 #' @references
 #'
@@ -141,7 +133,7 @@
 #'
 #' outliergram( fD, display = TRUE )
 #'
-#' outliergram( fD, Fvalue = 10, display = TRUE )
+#' outliergram( fD, Fvalue = 2.5, display = TRUE )
 #' \dontrun{
 #' outliergram( fD,
 #'              adjust = list( N_trials = 10,
@@ -154,10 +146,11 @@
 #' @importFrom grDevices dev.set dev.cur
 #' @importFrom stats cor pnorm rnorm qnorm uniroot
 #' @importFrom graphics text lines polygon plot points matplot par
+#' @importFrom dplyr filter group_by summarize
+#' @importFrom magrittr %>%
 #'
 #' @export
-outliergram = function( fData, MBD_data = NULL, MEI_data = NULL,
-                        q_low = 0, q_high = 1, p_check = 0.05,
+outliergram = function( fData, MBD_data = NULL, MEI_data = NULL, p_check = 0.05,
                         Fvalue = 1.5,
                         adjust = FALSE, display = TRUE,
                         xlab = NULL, ylab = NULL, main = NULL, ... )
@@ -171,21 +164,13 @@ outliergram = function( fData, MBD_data = NULL, MEI_data = NULL,
   if( ! is.list( adjust ) )
   {
     # Plain outliergram with default F value: F = 1.5
+    stopifnot( is.numeric(Fvalue) )
 
-    if( Fvalue == 1.5 )
-    {
-      out = .outliergram( fData,
-                          MBD_data = MBD_data, MEI_data = MEI_data,
-                          p_check = p_check, q_low = q_low, q_high = q_high,
-                          shift = TRUE )
-
-    } else {
-      out = .outliergram( fData,
-                          MBD_data = MBD_data, MEI_data = MEI_data,
-                          p_check = p_check, q_low = q_low, q_high = q_high,
-                          Fvalue = Fvalue,
-                          shift = TRUE )
-    }
+    out = .outliergram( fData,
+                        MBD_data = MBD_data, MEI_data = MEI_data,
+                        p_check = p_check,
+                        Fvalue = Fvalue,
+                        shift = TRUE )
   } else {
 
     nodenames = c( 'N_trials', 'trial_size', 'TPR', 'F_min', 'F_max',
@@ -248,8 +233,6 @@ outliergram = function( fData, MBD_data = NULL, MEI_data = NULL,
       .outliergram( fData_gauss,
                     MBD_data = NULL,
                     MEI_data = NULL,
-                    q_low = q_low,
-                    q_high = q_high,
                     Fvalue = F_curr,
                     shift = FALSE )$ID_SO ) / trial_size - TPR )
 
@@ -280,8 +263,7 @@ outliergram = function( fData, MBD_data = NULL, MEI_data = NULL,
 
     Fvalue = mean( Fvalues )
 
-    out = .outliergram( fData, MBD_data, MEI_data, p_check, q_low,
-                        q_high, Fvalue = Fvalue, shift = TRUE  )
+    out = .outliergram( fData, MBD_data, MEI_data, p_check, Fvalue = Fvalue, shift = TRUE  )
   }
 
   a_0_2 = -2 / ( N * ( N - 1 ) )
@@ -388,28 +370,19 @@ outliergram = function( fData, MBD_data = NULL, MEI_data = NULL,
     }
 
     # lower parabolic limit
-    if( Fvalue == 1.5 )
-    {
-      lines( grid_1D, a_0_2 + a_1 * grid_1D + a_0_2 * N^2 * grid_1D^2 -
-               out$Q_d3 - 1.5 * out$IQR_d,
-             lty = 2, lwd = 2, col = 'lightblue' )
-    }
-    else
-    {
-      lines( grid_1D,  a_0_2 + a_1 * grid_1D + a_0_2 * N^2 * grid_1D^2 -
-               Fvalue * out$Q_d1,
-             lty = 2, lwd = 2, col = 'lightblue' )
-    }
-
+    lines( grid_1D, a_0_2 + a_1 * grid_1D + a_0_2 * N^2 * grid_1D^2 -
+             out$Q_d3 - Fvalue * out$IQR_d,
+           lty = 2, lwd = 2, col = 'lightblue' )
   }
 
   return( list( Fvalue = Fvalue,
+                d = out$d,
                 ID_outliers = out$ID_SO ) )
 }
 
 .outliergram = function( fData, MBD_data = NULL, MEI_data = NULL,
-                         p_check = 0.05, q_low = 0, q_high = 1,
-                         Fvalue = NULL, shift = TRUE )
+                         p_check = 0.05,
+                         Fvalue = 1.5, shift = TRUE )
 {
   N = fData$N
 
@@ -440,117 +413,326 @@ outliergram = function( fData, MBD_data = NULL, MEI_data = NULL,
   IQR_d = Q[ 4 ] - Q[ 2 ]
 
   # Computing surely outlying curves
-
-  if( is.null( Fvalue ) )
-  {
-    ID_shape_outlier = which( d >= Q_d3 + 1.5 * IQR_d )
-  } else {
-
-    ID_shape_outlier = which( d >= Fvalue * Q_d1 )
-  }
+  ID_shape_outlier = which( d >= Q_d3 + Fvalue * IQR_d )
 
   # Computing non outlying curves ids
   ID_non_outlying = setdiff( 1 : nrow( fData$values ), ID_shape_outlier )
 
   if( shift )
   {
-    # # Low MEI curves will be checked for upward shift
-    # ID_non_outlying_Low_MEI = ID_non_outlying[
-    #   which( MEI_data[ - ID_shape_outlier ] >=
-    #            quantile( MEI_data,
-    #                      probs = 1 - p_check ) ) ]
-    #
-    # # High MEI curves will be checked for downward shift
-    # ID_non_outlying_High_MEI = ID_non_outlying[
-    #   which( MEI_data[ - ID_shape_outlier ] <=
-    #            quantile( MEI_data, probs = p_check ) ) ]
-
     # Low MEI curves will be checked for upward shift
-    ID_non_outlying_Low_MEI = ID_non_outlying[
-      which( MEI_data[ - ID_shape_outlier ] <=
-               quantile( MEI_data,
-                         probs = p_check ) ) ]
+      ID_non_outlying_Low_MEI = ID_non_outlying[
+        which( MEI_data[ - ID_shape_outlier ] <=
+                 quantile( MEI_data,
+                           probs = p_check ) ) ]
 
-    # High MEI curves will be checked for downward shift
-    ID_non_outlying_High_MEI = ID_non_outlying[
-      which( MEI_data[ - ID_shape_outlier ] >=
-               quantile( MEI_data, probs = 1 - p_check ) ) ]
+      # High MEI curves will be checked for downward shift
+      ID_non_outlying_High_MEI = ID_non_outlying[
+        which( MEI_data[ - ID_shape_outlier ] >=
+                 quantile( MEI_data, probs = 1 - p_check ) ) ]
 
 
-    aux_function = function( ID )( min( fData$values[ ID, ] -
-                                          apply( fData$values[ - ID, ], 2,
-                                                 quantile,
-                                                 probs = q_low ) ) )
+      # Manage high MEI data
+      lst = manage_high_MEI_data(fData = fData,
+                                 ID_non_outlying_High_MEI = ID_non_outlying_High_MEI,
+                                 ID_shape_outlier = ID_shape_outlier,
+                                 ID_non_outlying = ID_non_outlying,
+                                 Q_d1 = Q_d1,
+                                 Q_d3 = Q_d3,
+                                 IQR_d = IQR_d,
+                                 Fvalue = Fvalue)
 
-    # Managing High MEI data
-    min_diff_min = sapply( ID_non_outlying_High_MEI, aux_function )
+      ID_non_outlying = lst[['ID_non_outlying']]
+      ID_shape_outlier = lst[['ID_shape_outlier']]
 
-    ID_to_check = ID_non_outlying_High_MEI[ min_diff_min < 0 ]
+      # Manage low MEI data
+      lst = manage_low_MEI_data(fData = fData,
+                                ID_non_outlying_Low_MEI = ID_non_outlying_Low_MEI,
+                                ID_shape_outlier = ID_shape_outlier,
+                                ID_non_outlying = ID_non_outlying,
+                                Q_d1 = Q_d1,
+                                Q_d3 = Q_d3,
+                                IQR_d = IQR_d,
+                                Fvalue = Fvalue)
 
-    aux_function_MBD = function( ID )(
-      MBD( rbind( fData$values[ - ID, ],
-                  Data_tilde[ grep( ID, ID_to_check ), ] ) )[ N ] )
+      ID_non_outlying = lst[['ID_non_outlying']]
+      ID_shape_outlier = lst[['ID_shape_outlier']]
 
-    aux_function_MEI = function( ID )(
-      MEI( rbind( fData$values[ - ID, ],
-                  Data_tilde[ grep( ID, ID_to_check ), ] ) )[ N ] )
 
-    if( length( ID_to_check ) > 0 )
+  }
+
+  return( list( ID_SO = ID_shape_outlier,
+                ID_NO = ID_non_outlying,
+                MEI_data = MEI_data,
+                MBD_data = MBD_data,
+                Q_d3 = Q_d3,
+                Q_d1 = Q_d1,
+                IQR_d = IQR_d,
+                d = d) )
+
+}
+
+
+#' Outliergram for multivariate functional datasets
+#'
+#' This function performs the outliergram of a multivariate functional dataset.
+#'
+#' The method applies the extension of the univariate outliergram to the case of multivariate
+#' functional datasets. Differently from the function for the univariate case, only the
+#' outliergram plot is displayed.
+#'
+#' @section Adjustment:
+#'
+#' Differently from the case of univariate functional data, in this case the function does not apply
+#' an automatic tuning of the F parameter, since the related procedure would become computationally
+#' too heavy for general datasets. If a good value of F is sought, it is recommended to run several
+#' trials of the outliergram and manually select the best value.
+#'
+#' @param mfData the multivariate functional dataset whose outliergram has to be
+#' determined;
+#' @param MBD_data a vector containing the MBD for each element of the dataset; If missing, MBDs
+#' are computed with the specified choice of weights;
+#' @param weights the weights choice to be used to compute multivariate MBDs and MEIs;
+#' @param MEI_data a vector containing the MEI for each element of the dataset.
+#' If not not provided, MEIs are computed;
+#' @param p_check percentage of observations with either low or high MEI to be
+#' checked for outliers in the secondary step (shift towards the center of the
+#' dataset).
+#' @param Fvalue the \eqn{F} value to be used in the procedure that finds the
+#' shape outliers by looking at the lower parabolic limit in the outliergram.
+#' Default is \code{1.5};
+#' @param shift whether to apply the shifting algorithm to properly manage observations having low
+#' or high MEI. Default is TRUE.
+#' @param display either a logical value indicating wether you want the
+#' outliergram to be displayed, or the number of the graphical device
+#' where you want the outliergram to be displayed;
+#' @param xlab the label to use on the x axis in the outliergram plot;
+#' @param ylab the label to use on the x axis in the outliergram plot;
+#' @param main the title to use in the outliergram;
+#'
+#' @references
+#'
+#' Ieva, F. & Paganoni, A.M. Stat Papers (2017). https://doi.org/10.1007/s00362-017-0953-1.
+#'
+#' @seealso \code{\link{outliergram}}, \code{\link{mfData}}, \code{\link{MBD}},
+#' \code{\link{MEI}}
+#'
+#' @examples
+#'
+#' N = 2e2
+#' P = 1e2
+#'
+#' t0 = 0
+#' t1 = 1
+#'
+#' set.seed(1)
+#'
+#' # Defining the measurement grid
+#' grid = seq( t0, t1, length.out = P )
+#'
+#' # Generating an exponential covariance matrix to be used in the simulation of
+#' # the functional datasets (see the related help for details)
+#' C = exp_cov_function( grid, alpha = 0.3, beta = 0.2)
+#'
+#' # Simulating the measurements of two univariate functional datasets with
+#' # required center and covariance function
+#' f1 = function(x) x * ( 1 - x )
+#' f2 = function(x) x^3
+#' Data = generate_gauss_mfdata( N, L = 2,
+#'                               centerline = matrix(c(sin(2 * pi * grid),
+#'                                                     cos(2 * pi * grid)), nrow=2, byrow=TRUE),
+#'                               listCov = list(C, C), correlations = 0.1 )
+#'
+#' # Building the mfData object
+#' mfD = mfData( grid, Data )
+#'
+#'
+#' dev.new()
+#' out = multivariate_outliergram(mfD, Fvalue = 2., shift=TRUE)
+#' col_non_outlying = scales::hue_pal( h = c( 180, 270 ),
+#'                                     l = 60 )( N - length( out$ID_outliers ) )
+#' col_non_outlying = set_alpha( col_non_outlying, 0.5 )
+#' col_outlying = scales::hue_pal( h = c( - 90, 180  ),
+#'                                 c = 150 )( length( out$ID_outliers ) )
+#' colors = rep('black', N)
+#' colors[out$ID_outliers] = col_outlying
+#' colors[colors == 'black'] = col_non_outlying
+#'
+#' lwd = rep(1, N)
+#' lwd[out$ID_outliers] = 2
+#'
+#' dev.new()
+#' plot(mfD, col=colors, lwd=lwd)
+#'
+#' @importFrom dplyr filter group_by summarize
+#' @importFrom magrittr %>%
+#'
+#' @export
+multivariate_outliergram = function( mfData,
+                                     MBD_data = NULL,
+                                     MEI_data = NULL,
+                                     weights='uniform',
+                                     p_check = 0.05,
+                                     Fvalue = 1.5, shift = TRUE,
+                                     display = TRUE, xlab = NULL, ylab = NULL, main = NULL )
+{
+  N = mfData$N
+
+  out = .outliergram_mfData( mfData,
+                             MBD_data = MBD_data, MEI_data = MEI_data,
+                             p_check = p_check,
+                             Fvalue = Fvalue,
+                             shift = shift )
+
+  a_0_2 = -2 / ( N * ( N - 1 ) )
+
+  a_1 = 2 * ( N + 1 ) / ( N - 1 )
+
+  # Plot
+  if( display )
+  {
+
+    if( is.null( xlab ) )
     {
-      Data_tilde = toRowMatrixForm( fData$values[ ID_to_check, ] -
-                                      min_diff_min[ min_diff_min < 0 ] )
-
-      MBD_curr = sapply( ID_to_check, aux_function_MBD )
-
-      MEI_curr = sapply( ID_to_check, aux_function_MEI )
-
-      d_curr = a_0_2 + a_1 * MEI_curr + N^2 * a_0_2 * MEI_curr^2 - MBD_curr
-
-      if( is.null( Fvalue ) )
-      {
-        ID_out_extra = ID_to_check[ which( d_curr >= Q_d3 + 1.5 * IQR_d ) ]
-
-      } else {
-        ID_out_extra = ID_to_check[ which( d_curr >= Q_d1 * Fvalue ) ]
-      }
-
-      ID_shape_outlier = c( ID_shape_outlier, ID_out_extra )
-      ID_non_outlying = setdiff( ID_non_outlying, ID_out_extra )
+      xlab = 'MEI'
     }
 
-    # Managing Low MEI data
-
-    aux_function = function( ID )( max( fData$values[ ID, ] -
-                                          apply( fData$values[ - ID, ],
-                                                 2,
-                                                 quantile,
-                                                 probs = q_high ) ) )
-
-    max_diff_max = sapply( ID_non_outlying_Low_MEI, aux_function )
-
-    ID_to_check = ID_non_outlying_Low_MEI[ max_diff_max > 0 ]
-
-    if( length( ID_to_check ) > 0 )
+    if( is.null( ylab ) )
     {
-      Data_tilde = toRowMatrixForm( fData$values[ ID_to_check, ] -
-                                      max_diff_max[ max_diff_max > 0 ] )
+      ylab = 'MBD'
+    }
 
-      MBD_curr = sapply( ID_to_check, aux_function_MBD )
+    if( is.null( main ) )
+    {
+      main = 'Outliergram'
+    }
 
-      MEI_curr = sapply( ID_to_check, aux_function_MEI )
+    # Setting up palettes
+    col_non_outlying = scales::hue_pal( h = c( 180, 270 ),
+                                        l = 60 )( length( out$ID_NO ) )
 
-      d_curr = a_0_2 + a_1 * MEI_curr + N^2 * a_0_2 * MEI_curr^2 - MBD_curr
+    col_non_outlying = set_alpha( col_non_outlying, 0.5 )
 
-      if( is.null( Fvalue ) )
+    col_outlying = scales::hue_pal( h = c( - 90, 180  ),
+                                    c = 150 )( length( out$ID_SO ) )
+
+    dev.cur()
+    # Plotting outliergram
+    ## Upper parabolic limit
+    grid_1D = seq( 0, 1, length.out = 100 )
+
+    plot( grid_1D, a_0_2 + a_1 * grid_1D + a_0_2 * N^2 * grid_1D^2,
+          lty = 2, type = 'l', col = 'darkblue', lwd = 2,
+          ylim = c( 0, a_0_2 + a_1 / 2 + a_0_2 * N^2/4 ),
+          xlab = xlab,
+          ylab = ylab,
+          main = main )
+
+    if( length( out$ID_SO ) > 0 )
+    {
+      points( out$MEI_data[ - out$ID_SO ], out$MBD_data[ - out$ID_SO ],
+              pch = 16, col = col_non_outlying )
+      points( out$MEI_data[ out$ID_SO ], out$MBD_data[ out$ID_SO ],
+              pch = 16, cex = 1.5, col = col_outlying )
+      for( idOut in out$ID_SO )
       {
-        ID_out_extra = ID_to_check[ which( d_curr >= Q_d3 + 1.5 * IQR_d ) ]
-      } else {
-        ID_out_extra = ID_to_check[ which( d_curr >= Q_d1 * Fvalue ) ]
+        text( out$MEI_data[ idOut ],
+              out$MBD_data[ idOut ] + 0.5 / 30,
+              idOut,
+              col = col_outlying[ match( idOut, out$ID_SO ) ] )
       }
+    } else {
+      points( out$MEI_data, out$MBD_data,
+              pch = 16, col = col_non_outlying )
+    }
+
+    # lower parabolic limit
+      lines( grid_1D, a_0_2 + a_1 * grid_1D + a_0_2 * N^2 * grid_1D^2 -
+               out$Q_d3 - Fvalue * out$IQR_d,
+             lty = 2, lwd = 2, col = 'lightblue' )
+  }
+  return( list( Fvalue = Fvalue,
+                d = out$d,
+                ID_outliers = out$ID_SO ) )
+}
+
+.outliergram_mfData = function( mfData, MBD_data = NULL, MEI_data = NULL, weights='uniform',
+                                p_check = 0.05,
+                                Fvalue = 1.5, shift = TRUE )
+{
+  N = mfData$N
+
+  a_0_2 = -2 / ( N * ( N - 1 ) )
+
+  a_1 = 2 * ( N + 1 ) / ( N - 1 )
+
+  # Computing MBD
+  if( is.null( MBD_data ) ){
+    MBD_data = multiMBD( toListOfValues(mfData), weights = weights  )
+  }
+
+  # Computing MEI
+  if( is.null( MEI_data ) ){
+    MEI_data = multiMEI( toListOfValues(mfData), weights = weights )
+  }
+
+  d = a_0_2 + a_1 * MEI_data + N^2 * a_0_2 * MEI_data^2 - MBD_data
+
+  Q = quantile( d )
+
+  Q_d3 = Q[ 4 ]
+
+  Q_d1 = Q[ 2 ]
+
+  IQR_d = Q[ 4 ] - Q[ 2 ]
+
+  # Computing surely outlying curves
+  ID_shape_outlier = which( d >= Q_d3 + Fvalue * IQR_d )
+
+  # Computing non outlying curves ids
+  ID_non_outlying = setdiff( 1 : N, ID_shape_outlier )
+
+  if( shift )
+  {
+    for( iDim in seq_along(mfData$fDList))
+    {
+      # Low MEI curves will be checked for upward shift
+      ID_non_outlying_Low_MEI = ID_non_outlying[
+        which( MEI_data[ - ID_shape_outlier ] <=
+                 quantile( MEI_data,
+                           probs = p_check ) ) ]
+
+      # High MEI curves will be checked for downward shift
+      ID_non_outlying_High_MEI = ID_non_outlying[
+        which( MEI_data[ - ID_shape_outlier ] >=
+                 quantile( MEI_data, probs = 1 - p_check ) ) ]
 
 
-      ID_shape_outlier = c( ID_shape_outlier, ID_out_extra )
-      ID_non_outlying = setdiff( ID_non_outlying, ID_out_extra )
+      # Manage high MEI data
+      lst = manage_high_MEI_data(fData = mfData$fDList[[iDim]],
+                                 ID_non_outlying_High_MEI = ID_non_outlying_High_MEI,
+                                 ID_shape_outlier = ID_shape_outlier,
+                                 ID_non_outlying = ID_non_outlying,
+                                 Q_d1 = Q_d1,
+                                 Q_d3 = Q_d3,
+                                 IQR_d = IQR_d,
+                                 Fvalue = Fvalue)
+
+      ID_non_outlying = lst[['ID_non_outlying']]
+      ID_shape_outlier = lst[['ID_shape_outlier']]
+
+      # Manage low MEI data
+      lst = manage_low_MEI_data(fData = mfData$fDList[[iDim]],
+                                ID_non_outlying_Low_MEI = ID_non_outlying_Low_MEI,
+                                ID_shape_outlier = ID_shape_outlier,
+                                ID_non_outlying = ID_non_outlying,
+                                Q_d1 = Q_d1,
+                                Q_d3 = Q_d3,
+                                IQR_d = IQR_d,
+                                Fvalue = Fvalue)
+
+      ID_non_outlying = lst[['ID_non_outlying']]
+      ID_shape_outlier = lst[['ID_shape_outlier']]
     }
   }
 
@@ -560,7 +742,137 @@ outliergram = function( fData, MBD_data = NULL, MEI_data = NULL,
                 MBD_data = MBD_data,
                 Q_d3 = Q_d3,
                 Q_d1 = Q_d1,
-                IQR_d = IQR_d ) )
+                IQR_d = IQR_d,
+                d = d) )
 
 }
 
+
+manage_high_MEI_data = function(fData,
+                                ID_non_outlying_High_MEI,
+                                ID_shape_outlier,
+                                ID_non_outlying,
+                                Q_d1,
+                                Q_d3,
+                                IQR_d,
+                                Fvalue=1.5)
+{
+  N = fData$N
+
+  a_0_2 = -2 / ( N * ( N - 1 ) )
+  a_1 = 2 * ( N + 1 ) / ( N - 1 )
+
+  stopifnot( length(intersect(ID_shape_outlier, ID_non_outlying)) == 0 )
+
+  # Managing High MEI data
+  obs = min_diffs = NULL
+  mins = data.frame(min_diffs = apply(fData$values, 2, compute_min_diff_min),
+                    obs = apply(fData$values, 2, which.min))
+
+  min_diff_min = mins %>%
+    dplyr::filter(obs %in% ID_non_outlying_High_MEI ) %>%
+    dplyr::group_by(obs) %>%
+    dplyr::summarize(min_diff_min = min(min_diffs))
+
+  ID_to_check = as.list(min_diff_min)[['obs']]
+
+  aux_function_MBD = function( ID )(
+    MBD( rbind( fData$values[ - ID, ],
+                Data_tilde[ grep( ID, ID_to_check ), ] ) )[ N ] )
+
+  aux_function_MEI = function( ID )(
+    MEI( rbind( fData$values[ - ID, ],
+                Data_tilde[ grep( ID, ID_to_check ), ] ) )[ N ] )
+
+
+  if( nrow( min_diff_min ) > 0 )
+  {
+    Data_tilde = toRowMatrixForm( fData$values[ ID_to_check, ] +
+                                    as.list(min_diff_min)[['min_diff_min']])
+
+    MBD_curr = sapply( ID_to_check, aux_function_MBD )
+
+    MEI_curr = sapply( ID_to_check, aux_function_MEI )
+
+    d_curr = a_0_2 + a_1 * MEI_curr + N^2 * a_0_2 * MEI_curr^2 - MBD_curr
+
+    ID_out_extra = ID_to_check[ which( d_curr >= Q_d3 + Fvalue * IQR_d ) ]
+
+    ID_shape_outlier = c( ID_shape_outlier, ID_out_extra )
+    ID_non_outlying = setdiff( ID_non_outlying, ID_out_extra )
+  }
+
+  return( list(ID_shape_outlier = ID_shape_outlier,
+               ID_non_outlying = ID_non_outlying) )
+
+}
+
+manage_low_MEI_data = function(fData,
+                               ID_non_outlying_Low_MEI,
+                               ID_shape_outlier,
+                               ID_non_outlying,
+                               Q_d1,
+                               Q_d3,
+                               IQR_d,
+                               Fvalue = 1.5)
+{
+  N = fData$N
+
+  a_0_2 = -2 / ( N * ( N - 1 ) )
+  a_1 = 2 * ( N + 1 ) / ( N - 1 )
+
+  stopifnot( length(intersect(ID_shape_outlier, ID_non_outlying)) == 0 )
+
+  # Managing Low MEI data
+  obs = max_diffs = NULL
+  maxs = data.frame(max_diffs = apply(fData$values, 2, compute_max_diff_max),
+                    obs = apply(fData$values, 2, which.max))
+
+  max_diff_max = maxs %>%
+    dplyr::filter(obs %in% ID_non_outlying_Low_MEI ) %>%
+    dplyr::group_by(obs) %>%
+    dplyr::summarize(max_diff_max = max(max_diffs))
+
+  ID_to_check = as.list(max_diff_max)[['obs']]
+
+  aux_function_MBD = function( ID )(
+    MBD( rbind( fData$values[ - ID, ],
+                Data_tilde[ grep( ID, ID_to_check ), ] ) )[ N ] )
+
+  aux_function_MEI = function( ID )(
+    MEI( rbind( fData$values[ - ID, ],
+                Data_tilde[ grep( ID, ID_to_check ), ] ) )[ N ] )
+
+  if( nrow( max_diff_max ) > 0 )
+  {
+    Data_tilde = toRowMatrixForm( fData$values[ ID_to_check, ] +
+                                    as.list(max_diff_max)[['max_diff_max']])
+
+    MBD_curr = sapply( ID_to_check, aux_function_MBD )
+
+    MEI_curr = sapply( ID_to_check, aux_function_MEI )
+
+    d_curr = a_0_2 + a_1 * MEI_curr + N^2 * a_0_2 * MEI_curr^2 - MBD_curr
+
+    ID_out_extra = ID_to_check[ which( d_curr >= Q_d3 + Fvalue * IQR_d ) ]
+
+    ID_shape_outlier = c( ID_shape_outlier, ID_out_extra )
+    ID_non_outlying = setdiff( ID_non_outlying, ID_out_extra )
+  }
+
+  return( list(ID_shape_outlier = ID_shape_outlier,
+               ID_non_outlying = ID_non_outlying) )
+
+}
+
+
+compute_min_diff_min = function(x)
+{
+  return(diff(sort(x, partial=c(1,2), decreasing=FALSE)[c(1,2)]))
+}
+
+compute_max_diff_max = function(x)
+{
+  return(diff(-sort(-x, partial=c(1,2), decreasing=FALSE)[c(1,2)]))
+}
+
